package com.share.help.service;

import com.share.help.Util;
import com.share.help.dao.ActivityHistoryMapper;
import com.share.help.dao.ActivityMapper;
import com.share.help.dao.LeaveWordMapper;
import com.share.help.dao.UserMapper;
import com.share.help.entity.ActivityEntity;
import com.share.help.entity.ActivityHistoryEntity;
import com.share.help.entity.LeaveWordEntity;
import com.share.help.entity.UserEntity;
import com.share.help.entity.custorm.*;
import com.share.help.form.*;
import com.share.help.form.manager.ActiveUserRes;
import com.share.help.res.*;
import org.apache.commons.lang3.ArrayUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.sql.Timestamp;
import java.util.List;

/**
 * 活动服务层
 */
@Service
public class ActivityService{

    @Autowired
    private ActivityMapper activityMapper;

    @Autowired
    private ActivityHistoryMapper activityHistoryMapper;

    @Autowired
    private LeaveWordMapper leaveWordMapper;

    @Autowired
    private ImageService imageService;

    //活动分页配置
    @Value("${page-size.activity.page-size}")
    private int activityPageSize;

    @Value("${page-size.activity.row-size}")
    private int activityRowSize;

    @Value("${page-size.help}")
    private int helpPageSize;

    @Autowired
    private UserMapper userMapper;

    private Logger logger = LoggerFactory.getLogger(ActivityService.class);

    /**
     * 发布求助信息
     * @param sendHelpForm 求助信息表单
     * @param friendUserId 手动推荐好友
     * @return 发布结果
     */
    public JSONResponse<DefaultRes,Void> sendHelp(SendHelpForm sendHelpForm, String[] friendUserId){
        ActivityEntity activityEntity=new ActivityEntity();
        activityEntity.setUserId(sendHelpForm.getUserId());
        activityEntity.setTitle(sendHelpForm.getTitle());
        activityEntity.setContent(sendHelpForm.getContent());
        activityEntity.setSeekHelpTime(new Timestamp(System.currentTimeMillis()));
        activityEntity.setActivityStartTime(sendHelpForm.getActivityStartTime());
        activityEntity.setActivityEndTime(sendHelpForm.getActivityEndTime());
        //保存活动背景图
        String imageName=imageService.saveImg(sendHelpForm.getActivityImgFile());
        if(imageName==null){
            logger.error("活动背景图保存失败");
            return new JSONResponse<>(DefaultRes.fail, Result.FAIL);
        }
        activityEntity.setActivityImg(imageName);
        activityEntity.setServiceAddress(sendHelpForm.getServiceAddress());

        if(activityMapper.insert(activityEntity)) {
            switch (sendHelpForm.getRecommendType()) {
                case auto:
                    autoRecommend(activityEntity);
                    break;
                case choose:
                    if (ArrayUtils.isNotEmpty(friendUserId)) {
                        for (String userId : friendUserId) {
                            LeaveWordEntity leaveWordEntity = new LeaveWordEntity();
                            leaveWordEntity.setType(LeaveWordType.recommend.name());
                            leaveWordEntity.setSourceUserId(sendHelpForm.getUserId());
                            leaveWordEntity.setTargetUserId(userId);
                            leaveWordEntity.setCreateTime(new Timestamp(System.currentTimeMillis()));
                            leaveWordEntity.setActivityId(activityEntity.getActivityId());
                            leaveWordMapper.insert(leaveWordEntity);
                        }
                    }
                    break;
            }
            return new JSONResponse<>(DefaultRes.ok,Result.OK);
        }else{
            return new JSONResponse<>(DefaultRes.fail,Result.FAIL);
        }
    }



    /**
     * 系统推荐
     * @param activityEntity 推荐表单
     */
    private void autoRecommend(ActivityEntity activityEntity) {
        //查找参加过我活动的活跃好友
        List<ActiveUserRes> activeUserRes=activityMapper.queryActiveUser(activityEntity.getUserId(),activityEntity.getServiceAddress());
        for(ActiveUserRes res:activeUserRes){
            LeaveWordEntity leaveWordEntity = new LeaveWordEntity();
            leaveWordEntity.setType(LeaveWordType.recommend.name());
            leaveWordEntity.setSourceUserId(activityEntity.getUserId());
            leaveWordEntity.setTargetUserId(res.getUserId());
            leaveWordEntity.setCreateTime(new Timestamp(System.currentTimeMillis()));
            leaveWordEntity.setActivityId(activityEntity.getActivityId());
            if(leaveWordMapper.insert(leaveWordEntity)){
                logger.info("成功把活动id="+activityEntity.getActivityId()+"推荐给好友id="+res.getUserId());
            }
        }
    }


    /**
     * 分页查询活动信息
     * @param findActivityForm 查询活动信息
     * @return 返回活动信息
     */
    public JSONResponse<FindActivityRes.Res,FindActivityRes> find(FindActivityForm findActivityForm){
        FindActivityRes findActivityRes=activityMapper.count(activityPageSize,findActivityForm.getUserId(),findActivityForm.getTitle());
        findActivityRes.setPageSize(activityPageSize);
        findActivityRes.setCurrentPage(findActivityForm.getCurrentPage());
        List<ActivitySimple> activitySimpleList=activityMapper.getPage((findActivityForm.getCurrentPage()-1)* activityPageSize, activityPageSize,findActivityForm.getUserId(),findActivityForm.getTitle());
        if(!activitySimpleList.isEmpty()){
            findActivityRes.setBody(Util.splitList(ActivitySimple.class,activitySimpleList, activityRowSize));
            return new JSONResponse<FindActivityRes.Res,FindActivityRes>(FindActivityRes.Res.ok,Result.OK).setBody(findActivityRes);
        }else{
            return new JSONResponse<>(FindActivityRes.Res.empty, Result.OK);
        }
    }

    /**
     * 查看活动详情
     * @param activityId 活动id
     * @return 返回活动详情信息
     */
    public JSONResponse<DefaultRes,ActivityDetail> find(Long activityId,String userId){
        ActivityDetail activityDetail=activityMapper.findOne(activityId,userId);
        if(activityDetail==null){
            return new JSONResponse<>(DefaultRes.fail,Result.FAIL);
        }else{
            return new JSONResponse<DefaultRes,ActivityDetail>(DefaultRes.ok,Result.OK).setBody(activityDetail);
        }
    }

    /**
     * 报名活动
     * @param activityForm 报名名单
     * @return 返回报名结果
     */
    @Transactional
    public JSONResponse<DefaultRes,Integer> apply(UpdateActivityForm activityForm) throws RuntimeException{
        //记录用户完成活动返回的时间币
        int resultScore=0;
                
//        保存到活动历史记录
        ActivityHistoryEntity activityEntity=new ActivityHistoryEntity();
        activityEntity.setActivityId(activityForm.getActivityId());
        activityEntity.setUserId(activityForm.getUserId());
        activityEntity.setCreateTime(new Timestamp(System.currentTimeMillis()));
        activityEntity.setActivityStatus(activityForm.getActivityStatus().name());
        //完成活动结算时间币
        if(ActivityStatus.complete==activityForm.getActivityStatus()){
           int timeScore=activityHistoryMapper.getTimeScore(activityForm.getActivityId(),activityForm.getUserId());
           logger.info("奖励用户"+activityForm.getUserId()+"时间币："+timeScore);
           activityEntity.setTimeScore(timeScore);
           UserEntity userEntity=userMapper.findOne(activityEntity.getUserId());
            resultScore=timeScore+userEntity.getTimeScore();
           userEntity.setTimeScore(resultScore);
           
           if(!userMapper.update(userEntity)){
               throw new RuntimeException("结算时间币失败");
           }

            ActivitySendUser activitySendUser=activityMapper.querySendUser(activityForm.getActivityId());
            UserEntity sendUser=userMapper.findOne(activitySendUser.getUserId());
            logger.info("扣除用户"+activitySendUser.getUserId()+"时间币"+timeScore);
            sendUser.setTimeScore(sendUser.getTimeScore()-timeScore);

            if(!userMapper.update(sendUser)){
                throw new RuntimeException("扣除时间币失败");
            }
        }
        if(!activityHistoryMapper.insert(activityEntity)){
            throw new RuntimeException("保存活动历史记录失败");
        }

//        给求助用户发送留言
        LeaveWordEntity leaveWordEntity=new LeaveWordEntity();
        leaveWordEntity.setType(activityForm.getActivityStatus().name());
        leaveWordEntity.setSourceUserId(activityForm.getUserId());
        ActivityDetail activity=activityMapper.findOne(activityForm.getActivityId(),activityForm.getUserId());
        if(activity==null){
            throw new RuntimeException("查询活动记录失败");
        }
        leaveWordEntity.setTargetUserId(activity.getUserId());
        leaveWordEntity.setActivityId(activityForm.getActivityId());
        leaveWordEntity.setCreateTime(new Timestamp(System.currentTimeMillis()));
        if(!leaveWordMapper.insert(leaveWordEntity)){
            throw new RuntimeException("保存留言信息失败");
        }
        return new JSONResponse<DefaultRes,Integer>(DefaultRes.ok,Result.OK).setBody(resultScore);
    }

    /**
     * 求助信息统计
     * @return 返回统计结果
     */
    public JSONResponse<DefaultRes,Page<List<SeekHelpSum>>> seekHelpCount(SeekHelpForm seekHelpForm){
        Page<List<SeekHelpSum>> page=activityMapper.seekHelpCount(helpPageSize,seekHelpForm.getUserId());
        List<SeekHelpSum> seekHelpSums=activityMapper.seekHelp(seekHelpForm.getUserId(),(seekHelpForm.getCurrentPage()-1)*helpPageSize,helpPageSize);
        page.setBody(seekHelpSums);
        page.setCurrentPage(seekHelpForm.getCurrentPage());
        return new JSONResponse<DefaultRes,Page<List<SeekHelpSum>>>(DefaultRes.ok,Result.OK).setBody(page);
    }

    /**
     * 求助信息志愿者列表
     * @param findUserHelp 查询条件
     * @return 返回搜索结果
     */
    public JSONResponse<DefaultRes,List<UserWithName>> seekHelpList(FindUserHelp findUserHelp){
        List<UserWithName> userList=activityHistoryMapper.queryUser(findUserHelp.getActivityStatus().name(),findUserHelp.getActivityId());
        return new JSONResponse<DefaultRes,List<UserWithName>>(DefaultRes.ok,Result.OK).setBody(userList);
    }


    /**
     * 查询帮助列表
     * @param queryHelpForm 查询表单
     * @return 返回列表
     */
    public JSONResponse<DefaultRes,Page<List<HelpRes>>> queryHelp(QueryHelpForm queryHelpForm){
        Page<List<HelpRes>> page=activityHistoryMapper.count(queryHelpForm.getUserId(),helpPageSize);
        page.setCurrentPage(queryHelpForm.getCurrentPage());
        List<HelpRes> helpRes=activityHistoryMapper.queryMyHelp((queryHelpForm.getCurrentPage()-1)*helpPageSize,helpPageSize,queryHelpForm.getUserId());
        page.setBody(helpRes);
        return new JSONResponse<DefaultRes,Page<List<HelpRes>>>(DefaultRes.ok,Result.OK).setBody(page);
    }

    /**
     * 活动评价
     * @param helpScoreForm 评价表单
     * @return 返回评价结果
     */
    public JSONResponse<DefaultRes,Void> updateScore(HelpScoreForm helpScoreForm){
        if(activityHistoryMapper.updateScore(helpScoreForm.getHistoryId(),helpScoreForm.getScore())){
            return new JSONResponse<>(DefaultRes.ok,Result.OK);
        }else{
            return new JSONResponse<>(DefaultRes.fail,Result.FAIL);
        }
    }
}
