package com.share.help.service;

import com.share.help.Util;
import com.share.help.dao.FriendMapper;
import com.share.help.dao.UserMapper;
import com.share.help.entity.UserEntity;
import com.share.help.entity.custorm.FindUserRes;
import com.share.help.entity.custorm.Friend;
import com.share.help.entity.custorm.UserWithName;
import com.share.help.form.*;
import com.share.help.res.DefaultRes;
import com.share.help.res.JSONResponse;
import com.share.help.res.Page;
import com.share.help.res.Result;
import com.share.help.res.account.ModifyPwdRes;
import com.share.help.res.account.QueryRes;
import com.share.help.res.account.RegisterRes;
import com.share.help.res.account.UserLoginRes;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.DigestUtils;

import java.util.List;

/**
 * 用户服务层
 */
@Service
public class UserService{

    @Autowired
    private UserMapper userMapper;

    @Value("${user.register.time-score}")
    private int timeScore;

    //用户分页配置
    @Value("${page-size.user.page-size}")
    private int userPageSize;

    @Value("${page-size.user.row-size}")
    private int userRowSize;

    @Autowired
    private ImageService imageService;

    @Autowired
    private FriendMapper friendMapper;

    @Value("${page-size.friend}")
    private int friendSize;

    private Logger logger= LoggerFactory.getLogger(UserService.class);

    /**
     * 注册
     * @param userRegisterForm 注册表单
     * @return 返回注册结果
     */
    public JSONResponse<RegisterRes,Void> register(UserRegisterForm userRegisterForm){
//        用户id已存在，注册失败
        if(userMapper.hasUser(userRegisterForm.getUserId())){
            return new JSONResponse<>(RegisterRes.user_repeat,Result.FAIL);
        }

        UserEntity userEntity=new UserEntity();
        userEntity.setUserId(userRegisterForm.getUserId());
        //MD5密码
        String newPassword=DigestUtils.md5DigestAsHex(userRegisterForm.getPassword().getBytes());
        userEntity.setPassword(newPassword);
        userEntity.setName(userRegisterForm.getName());
        userEntity.setAge(userRegisterForm.getAge());
        userEntity.setSex(userRegisterForm.getSex());
        //保存头像
        String imageName=imageService.saveImg(userRegisterForm.getImgFile());
        if(imageName==null){
            logger.error("保存头像失败");
            return new JSONResponse<>(RegisterRes.fail, Result.FAIL);
        }
        userEntity.setHeadImg(imageName);
        userEntity.setAddress(userRegisterForm.getAddress());
        userEntity.setServiceAddress(userRegisterForm.getServiceAddress());
        userEntity.setUserType(userRegisterForm.getUserType());
        userEntity.setMobile(userRegisterForm.getMobile());
        userEntity.setEmail(userRegisterForm.getEmail());
        userEntity.setInfo(userRegisterForm.getInfo());
        userEntity.setTimeScore(timeScore);
        userEntity.setChatStatus(false);
        return userMapper.insert(userEntity)?new JSONResponse<>(RegisterRes.ok, Result.OK):new JSONResponse<>(RegisterRes.fail, Result.FAIL);
    }

    /**
     *
     * @param userLoginForm 登陆表单
     * @return 返回登录结果
     */
    public JSONResponse<UserLoginRes.Res, UserLoginRes> login(UserLoginForm userLoginForm){
        UserEntity userEntity=userMapper.findOneWithPwd(userLoginForm.getUserId(),DigestUtils.md5DigestAsHex(userLoginForm.getPassword().getBytes()));

        return userEntity==null?new JSONResponse<>(UserLoginRes.Res.valid_error,Result.FAIL)
                :new JSONResponse<UserLoginRes.Res, UserLoginRes>(UserLoginRes.Res.ok,Result.OK).setBody(new UserLoginRes(userEntity));
    }

    /**
     * 获取志愿者信息
     * @param findUserForm 搜索条件表单
     * @return 返回志愿者列表
     */
    public JSONResponse<FindUserRes.Res,FindUserRes> find(FindUserForm findUserForm){
        FindUserRes findUserRes=userMapper.count(userPageSize,findUserForm.getUserTypes(),findUserForm.getInfo(),findUserForm.getServiceAddress());
        findUserRes.setPageSize(userPageSize);
        findUserRes.setCurrentPage(findUserForm.getCurrentPage());
        List<UserEntity> userEntityList=userMapper.getPage((findUserForm.getCurrentPage()-1)* userPageSize, userPageSize,findUserForm.getUserTypes(),
                findUserForm.getInfo(),findUserForm.getServiceAddress());
        if(!userEntityList.isEmpty()){
            findUserRes.setBody(Util.splitList(UserEntity.class,userEntityList, userRowSize));
            return new JSONResponse<FindUserRes.Res,FindUserRes>(FindUserRes.Res.ok,Result.OK).setBody(findUserRes);
        }else{
            return new JSONResponse<>(FindUserRes.Res.empty, Result.OK);
        }
    }


    /**
     * 获取用户个人信息
     *
     * @param targetUserId 目标用户
     * @param userId 用户id
     * @return 返回用户信息
     */
    public JSONResponse<DefaultRes, QueryRes> find(String targetUserId, String userId){
        QueryRes queryRes= userMapper.findInfo(targetUserId,userId);
        if(queryRes!=null){
            return new JSONResponse<DefaultRes, QueryRes>(DefaultRes.ok,Result.OK).setBody(queryRes);
        }else{
            return new JSONResponse<>(DefaultRes.fail,Result.FAIL);
        }
    }

    /**
     * 更新用户信息
     * @param updateUserForm 用户id
     * @return 返回更新结果
     */
    public JSONResponse<DefaultRes,Void> update(UpdateUserForm updateUserForm){
        UserEntity userEntity=new UserEntity();
        userEntity.setUserId(updateUserForm.getUserId());
        userEntity.setName(updateUserForm.getName());
        userEntity.setAddress(updateUserForm.getAddress());
        userEntity.setServiceAddress(updateUserForm.getServiceAddress());
        userEntity.setUserType(updateUserForm.getUserType());
        userEntity.setMobile(updateUserForm.getMobile());
        userEntity.setEmail(updateUserForm.getEmail());
        userEntity.setInfo(updateUserForm.getInfo());
        userEntity.setAge(updateUserForm.getAge());
        userEntity.setSex(updateUserForm.getSex());

        if(updateUserForm.getActivityImgFile()!=null) {
            //保存头像
            String imageName = imageService.saveImg(updateUserForm.getActivityImgFile());
            if (imageName == null) {
                logger.error("更新头像失败");
                return new JSONResponse<>(DefaultRes.fail, Result.FAIL);
            }
            userEntity.setHeadImg(imageName);
        }

        if(userMapper.update(userEntity)){
            return new JSONResponse<>(DefaultRes.ok,Result.OK);
        }else{
            return new JSONResponse<>(DefaultRes.fail,Result.FAIL);
        }
    }

    /**
     * 更新用户密码
     * @param modifyPwdForm 更新密码表单
     * @return 返回更新结果
     */
    public JSONResponse<ModifyPwdRes,Void> update(ModifyPwdForm modifyPwdForm){
        UserEntity userEntity=userMapper.findOne(modifyPwdForm.getUserId());
        if(userEntity==null){
            return new JSONResponse<>(ModifyPwdRes.fail,Result.FAIL);
        }
        if(DigestUtils.md5DigestAsHex(modifyPwdForm.getOldPassword().getBytes()).equals(userEntity.getPassword())){
            userEntity.setPassword(DigestUtils.md5DigestAsHex(modifyPwdForm.getNewPassword().getBytes()));
            if(userMapper.update(userEntity)){
                return new JSONResponse<>(ModifyPwdRes.ok,Result.OK);
            }else{
                return new JSONResponse<>(ModifyPwdRes.fail,Result.FAIL);
            }
        }else{
            return new JSONResponse<>(ModifyPwdRes.pwd_fail,Result.FAIL);
        }
    }

    /**
     * 查询用户列表
     * @param queryUserForm 搜索条件
     * @return  返回查询结果
     */
    public JSONResponse<DefaultRes, Page<List<UserWithName>>> queryUser(QueryUserForm queryUserForm){
        Page<List<UserWithName>> page=userMapper.countUser(queryUserForm.getName(),queryUserForm.getInfo(),queryUserForm.getUserId(),queryUserForm.getTargetUserId(),userPageSize);
        List<UserWithName> userWithNameList=userMapper.queryUser(queryUserForm.getName(),queryUserForm.getInfo(),queryUserForm.getUserId(),queryUserForm.getTargetUserId());
        page.setBody(userWithNameList);
        return new JSONResponse<DefaultRes,Page<List<UserWithName>>>(DefaultRes.ok,Result.OK).setBody(page);
    }

    /**
     * 检索好友表单
     * @param searchFrom 检索条件
     * @return 返回好友列表
     */
    public JSONResponse<DefaultRes, Page<List<Friend>>> searchFriend(SearchFrom searchFrom) {
        Page<List<Friend>> page=friendMapper.count(searchFrom.getUserId(),searchFrom.getName(),searchFrom.getTargetUserId(),friendSize);
        page.setCurrentPage(searchFrom.getCurrentPage());
        List<Friend> list=friendMapper.getFriend(searchFrom.getUserId(),searchFrom.getName(),searchFrom.getTargetUserId(),(searchFrom.getCurrentPage()-1)*friendSize,friendSize);
        page.setBody(list);
        return new JSONResponse<DefaultRes,Page<List<Friend>>>(DefaultRes.ok,Result.OK).setBody(page);
    }

    /**
     * 刷新聊天状态
     * @param chatForm 表单
     * @return 返回刷新结果
     */
    public JSONResponse<DefaultRes, Void> changeChat(ChatForm chatForm) {
        UserEntity userEntity=userMapper.findOne(chatForm.getUserId());
        if(userEntity==null){
            return new JSONResponse<>(DefaultRes.fail,Result.FAIL);
        }
        userEntity.setChatStatus(chatForm.getChatStatus());
        if(userMapper.update(userEntity)){
            return new JSONResponse<>(DefaultRes.ok,Result.OK);
        }else{
            return new JSONResponse<>(DefaultRes.fail,Result.FAIL);
        }
    }
}
